/*
  GPS_Tester_N5IB.ino
  5 December 2019 
  Software serial multple serial test
  To test receiving from GPS, selecting a given NMEA sentence
  and parsing the sentence to extract relevant data
  and echo selected NMEA sentences to the serial monitor
  and display parsed data to an LCD panel

 Receives GPS serial data from software serial, sends to hardware serial.

 The circuit:
 * RX is digital pin connected to TX of other device)
 * TX is digital pin connected to RX of other device)

 created back in the mists of time
 modified 25 May 2012
 by Tom Igoe, based on Mikal Hart's example
 Modified 5 Dec 2019 for GPS specific serial data by Jim Giammanco, N5IB
 
 This example code is in the public domain.

 */

#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#define BACKLIGHT_PIN     3
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address, in this case hex 027

#define gpsRXpin 11   // RX is digital pin 11 (connect to TX of other device)
#define gpsTXpin 13   // TX is digital pin 13 (connect to RX of other device)
float Longitude, Lattitude;
long Latx, Lonx;
String GPS, MH_txt;


#include <SoftwareSerial.h>
SoftwareSerial gpsSerial(gpsRXpin, gpsTXpin); // create serial comms object

void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  Serial.println("Software Serial Test");

  // set the data rate for the SoftwareSerial port
  gpsSerial.begin(4800);

         lcd.begin(20, 4);  // configure for 20 X 4 display

  // Switch on the backlight
  pinMode ( BACKLIGHT_PIN, OUTPUT );
  digitalWrite ( BACKLIGHT_PIN, HIGH );

  lcd.setCursor(1,0);    
  lcd.print("    Hello N5IB!");
  delay(1000);
  lcd.home ();                   // go home
  lcd.clear();
}

void loop() { // run over and over

   GPS=gpsSerial.readStringUntil(0x0D); // wait for a $GPGGA sentence
   if (GPS.substring(1,7) == "$GPGGA")  {
     Serial.println(GPS);
     //lcd.clear();
     lcd.setCursor(0,0);   

     // display time as HH:MM  UTC
     lcd.print(GPS.substring(8,10));
     lcd.print(":"); 
     lcd.print(GPS.substring(10,12)); 
     lcd.print(" UTC");

     // display lattitude as decimal degrees
     int latDeg = GPS.substring(19,21).toInt();
     float latFrac = GPS.substring(21,28).toFloat();
     Lattitude = latDeg+latFrac/60;
     lcd.setCursor(0,1);  
     lcd.print(Lattitude,4);
     lcd.write(0xDF); //degree symbol
     lcd.print(GPS.substring(29,30));
         
     // display longitude as decimal degrees
     int lonDeg = GPS.substring(31,34).toInt();
     float lonFrac = GPS.substring(34,41).toFloat();
     Longitude=lonDeg+lonFrac/60;
     lcd.print(" ");
     lcd.print(Longitude,4);
     lcd.write(0xDF); //degree symbol
     lcd.print(GPS.substring(42,43));
     
     MaidenHead(); // decode grid square 6-character designator

     //display status of GPS fix
     lcd.setCursor(13,0);
     if (GPS.substring(44,45) != "0") lcd.print(MH_txt); // display grid locator if the fix is valid
     if (GPS.substring(44,45) == "0") lcd.print("NO FIX"); // other wise do not show grid

   } // end of $GPGGA processing

      GPS=gpsSerial.readStringUntil(0x0D); // wait for a $GPGGA sentence
   if (GPS.substring(1,7) == "$GPRMC")  {
     Serial.println(GPS);
     byte m = GPS.indexOf("W")+2;   // find the letter "W" in the string
     byte n = GPS.indexOf(",",m); // find the next comma, starting 2 characters after "W"
     float VOG = GPS.substring(m,n).toFloat();  // that's where the speed lies

     m = n+1; // move past the comma that follows the speed
     n = GPS.indexOf(",",m); // find the next comma
     float Course=GPS.substring(m,n).toFloat(); // get the TRUE heading
     lcd.setCursor(0,2);
     lcd.print(VOG*1.15078,1); // convert NMEA's knots to mph
     lcd.print("mph");
     lcd.setCursor(10,2);
     if (VOG > 0.4) {
        lcd.print(Course,0);
        lcd.write(0xDF); //degree symbol
        lcd.print(" True ");
     }
     else {
        lcd.print(" - - - - -");
     }
     
   } // end of $GPRMC processing
   
}

// routine to decode the 6-character Maidenhead gris locator based upon the current Lat/Lon
void MaidenHead(){
   // convert longitude into a long integer by multiplying by 100,000
   if (GPS.substring(42,43)=="W") Lonx = (-1)*Longitude*1e5;
      else Lonx=Longitude*1e5;
   Lonx = Lonx + 18000000;  // always positive east from Greenwich
   // Serial.print(Longitude); Serial.print("  "); Serial.println(Lonx);  // for debugging
   
   // convert lattitude into a long integer by multiplying by 100,000
   if (GPS.substring(29,30)=="S") Latx= (-1)*Lattitude*1e5;
      else Latx=Lattitude*1e5;
   Latx = Latx +  9000000; // Adjust so grid field AA is at the pole
   // Serial.print(Lattitude); Serial.print("  "); Serial.println(Latx);  // for debugging
   
   char MH[6] = {'A', 'A', '0', '0', 'a', 'a'};     // Initialise our print string
    MH[0] +=  Lonx / 2000000;                       // Field
    MH[1] +=  Latx / 1000000;
    MH[2] += (Lonx % 2000000) / 200000;             // Square
    MH[3] += (Latx % 1000000) / 100000;
    MH[4] += (Lonx %  200000) /   8333;             // Subsquare .08333 is   5/60 
    MH[5] += (Latx %  100000) /   4166;             //           .04166 is 2.5/60
  
   MH_txt = "";          // Build up Maidenhead, start with a null string
   int i = 0;            // add a character at a time, making a string that's easy to print
   while (i < 6){ 
      MH_txt += MH[i];
      i++; }

   Serial.print("Grid : ");
   Serial.println(MH_txt);

   return;
}

