/* **********************************************************
Serial_M)3_Command_tests_N5IB.ino     v1.00
Test program for the Catlex MP3 Player
Hardware: Serial MP3 Player
Board:  Arduino NANO
IDE:	  Arduino-1.8.5
Function:  Prompts user at Serial Monitor for folder number and file number, then plays that selection and waits for a new one
           Swiches digital ouput pins to control power to AF amp(s) - if used in hardware under test

Jim Giammanco, N5IB
22 January 2020
*/
#include <SoftwareSerial.h>   //  Arduino built-in library required for software serial port operation

#define mp3_RX 2  //NANO's D2 should connect to RX pin of the Serial MP3 Player module
#define mp3_TX 3  //NANO's D3 should connect to TX pin of the module  (not actually used, but connected)
#define MP3_baud 9600   // data rate for serial data sent to MP3 player (should not have to be changed)
SoftwareSerial mp3Serial(mp3_TX, mp3_RX);     // create serial comms for MP3, set up RX and TX lines for the MP3 player
                                              // one directional data only, only mp3RX needed
                                              
// assign these two pins that control power to the audio amp(s) if testing hardware that implements his feature
#define AFpwrR 10 // digital output to control power to right channel AF amp
#define AFpwrL 9  // digital output to control power to left channel AF amp; or spare digital I/O

// define the MP3 player's single byte command codes
#define SINGLE_CYCLE_ON 0X00
#define SINGLE_CYCLE_OFF 0X01
#define DEV_TF 0X02
#define CMD_PLAY_W_INDEX 0X03
#define CMD_LOUDER 0X04
#define CMD_SOFTER 0X05
#define CMD_SET_VOLUME 0X06
#define CMD_SEL_DEV 0X09
#define CMD_SLEEP 0X0A
#define CMD_WAKEUP 0X0B
#define CMD_RESET 0X0C
#define CMD_PLAY 0X0D
#define CMD_PAUSE 0X0E
#define CMD_PLAY_W_FOLDER_FILENAME 0X0F
#define CMD_STOP 0X16
#define CMD_CYCLEPLAY_W_FOLDERNAME 0X17
#define CMD_SHUFFLE_PLAY 0X18
#define CMD_SINGLE_CYCLE 0X19
#define CMD_DAC 0X1A
#define CMD_PLAY_W_VOL 0X22

byte MP3Command, MP3DataH, MP3DataL; // arguments for "sendCommand" function
byte selDir, selFile;                // directory and file selection for MP3 file
static int8_t Send_buf[8] = {0} ;    // transmit buffer for MP3 commands

void setup() 
{
	 Serial.begin(9600) ;
   Serial.setTimeout(60000);  // wait up to a minute for some keyboard input
   mp3Serial.begin(9600);
	 delay(500);//Wait until chip initialization is complete
   mp3Serial.begin(MP3_baud);  // set the data rate for thee MP3 software serial port
   delay(500);//Wait until chip initialization is complete
   sendCommand(CMD_SEL_DEV, 0x00, DEV_TF);//select the TF card  
   delay(200);//wait for 200ms
   sendCommand(CMD_SET_VOLUME, 0x00, 0x13);//set the playback volume,hex001E is full volume
   delay(200);//wait for 200ms
   Serial.println("starting MP3 Player Test Program v1.00") ;

// if this sketch is being used to test the circuit which uses a transistor switch to control power to one or two audio amplifiers
// the following three lines serve to turn on the amplifier(s)
// the headphone output of the MP3 module is always on, and is not affected by this process
   
   pinMode (AFpwrR,OUTPUT);     // drive this pin low to power up the right channel AF amp
   pinMode (AFpwrL,OUTPUT);     // drive this pin low to power up the left channel AF amp
   digitalWrite(AFpwrR,LOW); digitalWrite(AFpwrL, LOW);  // turn ON both the AF amps, if used in this test

}   //  end of setup sequence

void loop() 
{  
  while (Serial.available() > 0) Serial.read();  // clear the serial buffer
  Serial.print("Enter folder number, starting at 1 :");
  selDir = Serial.parseInt();
  Serial.println(selDir);

  while (Serial.available() > 0) Serial.read();  // clear the serial buffer
  Serial.print("Enter file number, starting at 001 :");
  selFile = Serial.parseInt();
  Serial.println(selFile);
  sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir, selFile); //   play requested folder/file

  delay(5000);                
}    // end of loop sequence

// routine to transmit a 7-byte command sequence serially to the MP3 module
void sendCommand(byte P1, byte P2, byte P3)
{
  delay(20);
  Send_buf[0] = 0x7e;               //starting byte
  Send_buf[1] = 0xff;               //version
  Send_buf[2] = 0x06;               //the number of bytes of the command without starting byte and ending byte
  Send_buf[3] = P1;                 // actual command code
  Send_buf[4] = 0x00;               //0x00 = no feedback, 0x01 = feedback  --- feedback not used here
  Send_buf[5] = P2;                 //data-most significant byte
  Send_buf[6] = P3;                 //data-least significant byte
  Send_buf[7] = 0xef;               //ending byte
  for(byte i=0; i<8; i++)
  {
    mp3Serial.write(Send_buf[i]) ;  // transmit one byte at a time
  }
}  // end of command transmission routine

